@@ -9,15 +9,13 @@ class TestFileFpx(PillowTestCase):
         # Test an invalid OLE file
         invalid_file = "Tests/images/flower.jpg"
         self.assertRaises(SyntaxError,
-                          lambda: FpxImagePlugin.FpxImageFile(invalid_file))
+                          FpxImagePlugin.FpxImageFile, invalid_file)
 
         # Test a valid OLE file, but not an FPX file
         ole_file = "Tests/images/test-ole-file.doc"
         self.assertRaises(SyntaxError,
-                          lambda: FpxImagePlugin.FpxImageFile(ole_file))
+                          FpxImagePlugin.FpxImageFile, ole_file)
 
 
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
