@@ -31,7 +31,7 @@ class TestFileGif(PillowTestCase):
         invalid_file = "Tests/images/flower.jpg"
 
         self.assertRaises(SyntaxError,
-                          lambda: GifImagePlugin.GifImageFile(invalid_file))
+                          GifImagePlugin.GifImageFile, invalid_file)
 
     def test_optimize(self):
         def test_grayscale(optimize):
@@ -208,25 +208,29 @@ class TestFileGif(PillowTestCase):
             self.assertEqual(framecount, 5)
 
     def test_n_frames(self):
-        im = Image.open(TEST_GIF)
-        self.assertEqual(im.n_frames, 1)
-        self.assertFalse(im.is_animated)
+        for path, n_frames in [
+            [TEST_GIF, 1],
+            ['Tests/images/iss634.gif', 42]
+        ]:
+            # Test is_animated before n_frames
+            im = Image.open(path)
+            self.assertEqual(im.is_animated, n_frames != 1)
 
-        im = Image.open("Tests/images/iss634.gif")
-        self.assertEqual(im.n_frames, 42)
-        self.assertTrue(im.is_animated)
+            # Test is_animated after n_frames
+            im = Image.open(path)
+            self.assertEqual(im.n_frames, n_frames)
+            self.assertEqual(im.is_animated, n_frames != 1)
 
     def test_eoferror(self):
         im = Image.open(TEST_GIF)
-
         n_frames = im.n_frames
-        while True:
-            n_frames -= 1
-            try:
-                im.seek(n_frames)
-                break
-            except EOFError:
-                self.assertLess(im.tell(), n_frames)
+
+        # Test seeking past the last frame
+        self.assertRaises(EOFError, im.seek, n_frames)
+        self.assertLess(im.tell(), n_frames)
+
+        # Test that seeking to the last frame does not raise an error
+        im.seek(n_frames-1)
 
     def test_dispose_none(self):
         img = Image.open("Tests/images/dispose_none.gif")
@@ -375,29 +379,31 @@ class TestFileGif(PillowTestCase):
     def test_version(self):
         out = self.tempfile('temp.gif')
 
+        def assertVersionAfterSave(im, version):
+            im.save(out)
+            reread = Image.open(out)
+            self.assertEqual(reread.info["version"], version)
+
         # Test that GIF87a is used by default
         im = Image.new('L', (100, 100), '#000')
-        im.save(out)
-        reread = Image.open(out)
-        self.assertEqual(reread.info["version"], b"GIF87a")
+        assertVersionAfterSave(im, b"GIF87a")
+
+        # Test setting the version to 89a
+        im = Image.new('L', (100, 100), '#000')
+        im.info["version"] = b"89a"
+        assertVersionAfterSave(im, b"GIF89a")
 
         # Test that adding a GIF89a feature changes the version
         im.info["transparency"] = 1
-        im.save(out)
-        reread = Image.open(out)
-        self.assertEqual(reread.info["version"], b"GIF89a")
+        assertVersionAfterSave(im, b"GIF89a")
 
         # Test that a GIF87a image is also saved in that format
         im = Image.open("Tests/images/test.colors.gif")
-        im.save(out)
-        reread = Image.open(out)
-        self.assertEqual(reread.info["version"], b"GIF87a")
+        assertVersionAfterSave(im, b"GIF87a")
 
         # Test that a GIF89a image is also saved in that format
         im.info["version"] = b"GIF89a"
-        im.save(out)
-        reread = Image.open(out)
-        self.assertEqual(reread.info["version"], b"GIF87a")
+        assertVersionAfterSave(im, b"GIF87a")
 
     def test_append_images(self):
         out = self.tempfile('temp.gif')
