@@ -40,13 +40,15 @@ class SimplePatcher(object):
         else:
             delattr(self._parent_obj, self._attr_name)
 
+
 @unittest.skipUnless(HAS_FREETYPE, "ImageFont not Available")
 class TestImageFont(PillowTestCase):
     LAYOUT_ENGINE = ImageFont.LAYOUT_BASIC
 
     # Freetype has different metrics depending on the version.
     # (and, other things, but first things first)
-    METRICS = { ('2', '3'): {'multiline': 30,
+    METRICS = {
+                ('2', '3'): {'multiline': 30,
                              'textsize': 12,
                              'getters': (13, 16)},
                 ('2', '7'): {'multiline': 6.2,
@@ -63,11 +65,11 @@ class TestImageFont(PillowTestCase):
     def setUp(self):
         freetype_version = tuple(ImageFont.core.freetype2_version.split('.'))[:2]
         self.metrics = self.METRICS.get(freetype_version, self.METRICS['Default'])
-    
+
     def get_font(self):
         return ImageFont.truetype(FONT_PATH, FONT_SIZE,
                                   layout_engine=self.LAYOUT_ENGINE)
-    
+
     def test_sanity(self):
         self.assertRegexpMatches(
             ImageFont.core.freetype2_version, r"\d+\.\d+\.\d+$")
@@ -104,7 +106,7 @@ class TestImageFont(PillowTestCase):
         # Usage note:  making two fonts from the same buffer fails.
         # shared_bytes = self._font_as_bytes()
         # self._render(shared_bytes)
-        # self.assertRaises(Exception, lambda: _render(shared_bytes))
+        # self.assertRaises(Exception, _render, shared_bytes)
 
     def test_font_with_open_file(self):
         with open(FONT_PATH, 'rb') as f:
@@ -139,7 +141,6 @@ class TestImageFont(PillowTestCase):
         size = draw.textsize(txt, ttf)
         draw.text((10, 10), txt, font=ttf)
         draw.rectangle((10, 10, 10 + size[0], 10 + size[1]))
-        del draw
 
         target = 'Tests/images/rectangle_surrounding_text.png'
         target_img = Image.open(target)
@@ -186,7 +187,6 @@ class TestImageFont(PillowTestCase):
         draw.text((0, 0), TEST_TEXT, fill=None, font=ttf, anchor=None,
                   spacing=4, align="left")
         draw.text((0, 0), TEST_TEXT, None, ttf, None, 4, "left")
-        del draw
 
         # Test align center and right
         for align, ext in {"center": "_center",
@@ -194,7 +194,6 @@ class TestImageFont(PillowTestCase):
             im = Image.new(mode='RGB', size=(300, 100))
             draw = ImageDraw.Draw(im)
             draw.multiline_text((0, 0), TEST_TEXT, font=ttf, align=align)
-            del draw
 
             target = 'Tests/images/multiline_text'+ext+'.png'
             target_img = Image.open(target)
@@ -209,9 +208,16 @@ class TestImageFont(PillowTestCase):
 
         # Act/Assert
         self.assertRaises(AssertionError,
-                          lambda: draw.multiline_text((0, 0), TEST_TEXT,
-                                                      font=ttf,
-                                                      align="unknown"))
+                          draw.multiline_text, (0, 0), TEST_TEXT,
+                                                       font=ttf,
+                                                       align="unknown")
+
+    def test_draw_align(self):
+        im = Image.new('RGB', (300, 100), 'white')
+        draw = ImageDraw.Draw(im)
+        ttf = self.get_font()
+        line = "some text"
+        draw.text((100, 40), line, (0, 0, 0), font=ttf, align='left')
 
     def test_multiline_size(self):
         ttf = self.get_font()
@@ -226,7 +232,6 @@ class TestImageFont(PillowTestCase):
         # to multiline_textsize()
         draw.textsize(TEST_TEXT, font=ttf, spacing=4)
         draw.textsize(TEST_TEXT, ttf, 4)
-        del draw
 
     def test_multiline_width(self):
         ttf = self.get_font()
@@ -236,7 +241,6 @@ class TestImageFont(PillowTestCase):
         self.assertEqual(draw.textsize("longest line", font=ttf)[0],
                          draw.multiline_textsize("longest line\nline",
                                                  font=ttf)[0])
-        del draw
 
     def test_multiline_spacing(self):
         ttf = self.get_font()
@@ -244,13 +248,10 @@ class TestImageFont(PillowTestCase):
         im = Image.new(mode='RGB', size=(300, 100))
         draw = ImageDraw.Draw(im)
         draw.multiline_text((0, 0), TEST_TEXT, font=ttf, spacing=10)
-        del draw
 
         target = 'Tests/images/multiline_text_spacing.png'
         target_img = Image.open(target)
-        from PIL import ImageChops
-        
-        ImageChops.difference(im, target_img).save('multiline.png')
+
         # Epsilon ~.5 fails with FreeType 2.7
         self.assert_image_similar(im, target_img, self.metrics['multiline'])
 
@@ -271,7 +272,6 @@ class TestImageFont(PillowTestCase):
         # Rotated font
         draw.font = transposed_font
         box_size_b = draw.textsize(word)
-        del draw
 
         # Check (w,h) of box a is (h,w) of box b
         self.assertEqual(box_size_a[0], box_size_b[1])
@@ -294,7 +294,6 @@ class TestImageFont(PillowTestCase):
         # Rotated font
         draw.font = transposed_font
         box_size_b = draw.textsize(word)
-        del draw
 
         # Check boxes a and b are same size
         self.assertEqual(box_size_a, box_size_b)
@@ -376,7 +375,7 @@ class TestImageFont(PillowTestCase):
         filename = "somefilenamethatdoesntexist.ttf"
 
         # Act/Assert
-        self.assertRaises(IOError, lambda: ImageFont.load_path(filename))
+        self.assertRaises(IOError, ImageFont.load_path, filename)
 
     def test_default_font(self):
         # Arrange
@@ -390,11 +389,29 @@ class TestImageFont(PillowTestCase):
         # Act
         default_font = ImageFont.load_default()
         draw.text((10, 10), txt, font=default_font)
-        del draw
 
         # Assert
         self.assert_image_equal(im, target_img)
 
+    def test_getsize_empty(self):
+        # issue #2614
+        font = self.get_font()
+        # should not crash.
+        self.assertEqual((0, 0), font.getsize(''))
+
+    def test_render_empty(self):
+        # issue 2666
+        font = self.get_font()
+        im = Image.new(mode='RGB', size=(300, 100))
+        target = im.copy()
+        draw = ImageDraw.Draw(im)
+        #should not crash here. 
+        draw.text((10, 10), '', font=font)
+        self.assert_image_equal(im, target)
+        
+       
+        
+
     def _test_fake_loading_font(self, path_to_fake, fontname):
         # Make a copy of FreeTypeFont so we can patch the original
         free_type_font = copy.deepcopy(ImageFont.FreeTypeFont)
@@ -491,5 +508,6 @@ class TestImageFont(PillowTestCase):
 class TestImageFont_RaqmLayout(TestImageFont):
     LAYOUT_ENGINE = ImageFont.LAYOUT_RAQM
 
+
 if __name__ == '__main__':
     unittest.main()
