@@ -599,7 +599,7 @@ _fill(PyObject* self, PyObject* args)
     if (!PyArg_ParseTuple(args, "s|(ii)O", &mode, &xsize, &ysize, &color))
         return NULL;
 
-    im = ImagingNew(mode, xsize, ysize);
+    im = ImagingNewDirty(mode, xsize, ysize);
     if (!im)
         return NULL;
 
@@ -641,15 +641,6 @@ _new_block(PyObject* self, PyObject* args)
     return PyImagingNew(ImagingNewBlock(mode, xsize, ysize));
 }
 
-static PyObject*
-_getcount(PyObject* self, PyObject* args)
-{
-    if (!PyArg_ParseTuple(args, ":getcount"))
-        return NULL;
-
-    return PyInt_FromLong(ImagingNewCount);
-}
-
 static PyObject*
 _linear_gradient(PyObject* self, PyObject* args)
 {
@@ -791,23 +782,6 @@ _copy(ImagingObject* self, PyObject* args)
     return PyImagingNew(ImagingCopy(self->image));
 }
 
-static PyObject*
-_copy2(ImagingObject* self, PyObject* args)
-{
-    ImagingObject* imagep1;
-    ImagingObject* imagep2;
-    if (!PyArg_ParseTuple(args, "O!O!",
-                          &Imaging_Type, &imagep1,
-                          &Imaging_Type, &imagep2))
-        return NULL;
-
-    if (!ImagingCopy2(imagep1->image, imagep2->image))
-        return NULL;
-
-    Py_INCREF(Py_None);
-    return Py_None;
-}
-
 static PyObject*
 _crop(ImagingObject* self, PyObject* args)
 {
@@ -836,7 +810,7 @@ _filter(ImagingObject* self, PyObject* args)
     Py_ssize_t kernelsize;
     FLOAT32* kerneldata;
 
-    int xsize, ysize;
+    int xsize, ysize, i;
     float divisor, offset;
     PyObject* kernel = NULL;
     if (!PyArg_ParseTuple(args, "(ii)ffO", &xsize, &ysize,
@@ -852,8 +826,12 @@ _filter(ImagingObject* self, PyObject* args)
         return ImagingError_ValueError("bad kernel size");
     }
 
+    for (i = 0; i < kernelsize; ++i) {
+        kerneldata[i] /= divisor;
+    }
+
     imOut = PyImagingNew(
-        ImagingFilter(self->image, xsize, ysize, kerneldata, offset, divisor)
+        ImagingFilter(self->image, xsize, ysize, kerneldata, offset)
         );
 
     free(kerneldata);
@@ -874,7 +852,7 @@ _gaussian_blur(ImagingObject* self, PyObject* args)
         return NULL;
 
     imIn = self->image;
-    imOut = ImagingNew(imIn->mode, imIn->xsize, imIn->ysize);
+    imOut = ImagingNewDirty(imIn->mode, imIn->xsize, imIn->ysize);
     if (!imOut)
         return NULL;
 
@@ -1450,7 +1428,7 @@ _putpalettealphas(ImagingObject* self, PyObject* args)
     int i;
     UINT8 *values;
     int length;
-    if (!PyArg_ParseTuple(args, "s#", &values, &length))
+    if (!PyArg_ParseTuple(args, PY_ARG_BYTES_LENGTH, &values, &length))
         return NULL;
 
     if (!self->image->palette) {
@@ -1520,26 +1498,47 @@ _resize(ImagingObject* self, PyObject* args)
 
     int xsize, ysize;
     int filter = IMAGING_TRANSFORM_NEAREST;
-    if (!PyArg_ParseTuple(args, "(ii)|i", &xsize, &ysize, &filter))
-        return NULL;
-
+    float box[4] = {0, 0, 0, 0};
+    
     imIn = self->image;
+    box[2] = imIn->xsize;
+    box[3] = imIn->ysize;
+    
+    if (!PyArg_ParseTuple(args, "(ii)|i(ffff)", &xsize, &ysize, &filter,
+                          &box[0], &box[1], &box[2], &box[3]))
+        return NULL;
 
     if (xsize < 1 || ysize < 1) {
         return ImagingError_ValueError("height and width must be > 0");
     }
 
-    if (imIn->xsize == xsize && imIn->ysize == ysize) {
-        imOut = ImagingCopy(imIn);
+    if (box[0] < 0 || box[1] < 0) {
+        return ImagingError_ValueError("box offset can't be negative");
+    }
+
+    if (box[2] > imIn->xsize || box[3] > imIn->ysize) {
+        return ImagingError_ValueError("box can't exceed original image size");
+    }
+
+    if (box[2] - box[0] < 0 || box[3] - box[1] < 0) {
+        return ImagingError_ValueError("box can't be empty");
+    }
+
+    // If box's coordinates are int and box size matches requested size
+    if (box[0] - (int) box[0] == 0 && box[2] - box[0] == xsize
+            && box[1] - (int) box[1] == 0 && box[3] - box[1] == ysize) {
+        imOut = ImagingCrop(imIn, box[0], box[1], box[2], box[3]);
     }
     else if (filter == IMAGING_TRANSFORM_NEAREST) {
         double a[6];
 
         memset(a, 0, sizeof a);
-        a[0] = (double) imIn->xsize / xsize;
-        a[4] = (double) imIn->ysize / ysize;
+        a[0] = (double) (box[2] - box[0]) / xsize;
+        a[4] = (double) (box[3] - box[1]) / ysize;
+        a[2] = box[0];
+        a[5] = box[1];
 
-        imOut = ImagingNew(imIn->mode, xsize, ysize);
+        imOut = ImagingNewDirty(imIn->mode, xsize, ysize);
 
         imOut = ImagingTransform(
             imOut, imIn, IMAGING_TRANSFORM_AFFINE,
@@ -1547,7 +1546,7 @@ _resize(ImagingObject* self, PyObject* args)
             a, filter, 1);
     }
     else {
-        imOut = ImagingResample(imIn, xsize, ysize, filter);
+        imOut = ImagingResample(imIn, xsize, ysize, filter, box);
     }
 
     return PyImagingNew(imOut);
@@ -1665,12 +1664,13 @@ _transpose(ImagingObject* self, PyObject* args)
     case 0: /* flip left right */
     case 1: /* flip top bottom */
     case 3: /* rotate 180 */
-        imOut = ImagingNew(imIn->mode, imIn->xsize, imIn->ysize);
+        imOut = ImagingNewDirty(imIn->mode, imIn->xsize, imIn->ysize);
         break;
     case 2: /* rotate 90 */
     case 4: /* rotate 270 */
     case 5: /* transpose */
-        imOut = ImagingNew(imIn->mode, imIn->ysize, imIn->xsize);
+    case 6: /* transverse */
+        imOut = ImagingNewDirty(imIn->mode, imIn->ysize, imIn->xsize);
         break;
     default:
         PyErr_SetString(PyExc_ValueError, "No such transpose operation");
@@ -1697,6 +1697,9 @@ _transpose(ImagingObject* self, PyObject* args)
         case 5:
             (void) ImagingTranspose(imOut, imIn);
             break;
+        case 6:
+            (void) ImagingTransverse(imOut, imIn);
+            break;
         }
 
     return PyImagingNew(imOut);
@@ -1715,7 +1718,7 @@ _unsharp_mask(ImagingObject* self, PyObject* args)
         return NULL;
 
     imIn = self->image;
-    imOut = ImagingNew(imIn->mode, imIn->xsize, imIn->ysize);
+    imOut = ImagingNewDirty(imIn->mode, imIn->xsize, imIn->ysize);
     if (!imOut)
         return NULL;
 
@@ -1738,7 +1741,7 @@ _box_blur(ImagingObject* self, PyObject* args)
         return NULL;
 
     imIn = self->image;
-    imOut = ImagingNew(imIn->mode, imIn->xsize, imIn->ysize);
+    imOut = ImagingNewDirty(imIn->mode, imIn->xsize, imIn->ysize);
     if (!imOut)
         return NULL;
 
@@ -1753,7 +1756,7 @@ _box_blur(ImagingObject* self, PyObject* args)
 static PyObject*
 _isblock(ImagingObject* self, PyObject* args)
 {
-    return PyInt_FromLong((long) self->image->block);
+    return PyBool_FromLong(self->image->block != NULL);
 }
 
 static PyObject*
@@ -1905,6 +1908,55 @@ _putband(ImagingObject* self, PyObject* args)
     return Py_None;
 }
 
+static PyObject*
+_merge(PyObject* self, PyObject* args)
+{
+    char* mode;
+    ImagingObject *band0 = NULL;
+    ImagingObject *band1 = NULL;
+    ImagingObject *band2 = NULL;
+    ImagingObject *band3 = NULL;
+    Imaging bands[4] = {NULL, NULL, NULL, NULL};
+
+    if (!PyArg_ParseTuple(args, "sO!|O!O!O!", &mode,
+                          &Imaging_Type, &band0, &Imaging_Type, &band1,
+                          &Imaging_Type, &band2, &Imaging_Type, &band3))
+        return NULL;
+
+    if (band0) bands[0] = band0->image;
+    if (band1) bands[1] = band1->image;
+    if (band2) bands[2] = band2->image;
+    if (band3) bands[3] = band3->image;
+
+    return PyImagingNew(ImagingMerge(mode, bands));
+}
+
+static PyObject*
+_split(ImagingObject* self, PyObject* args)
+{
+    int fails = 0;
+    Py_ssize_t i;
+    PyObject* list;
+    PyObject* imaging_object;
+    Imaging bands[4] = {NULL, NULL, NULL, NULL};
+
+    if ( ! ImagingSplit(self->image, bands))
+        return NULL;
+
+    list = PyTuple_New(self->image->bands);
+    for (i = 0; i < self->image->bands; i++) {
+        imaging_object = PyImagingNew(bands[i]);
+        if ( ! imaging_object)
+            fails += 1;
+        PyTuple_SET_ITEM(list, i, imaging_object);
+    }
+    if (fails) {
+        Py_DECREF(list);
+        list = NULL;
+    }
+    return list;
+}
+
 /* -------------------------------------------------------------------- */
 
 #ifdef WITH_IMAGECHOPS
@@ -2145,26 +2197,45 @@ textwidth(ImagingFontObject* self, const unsigned char* text)
 }
 
 void _font_text_asBytes(PyObject* encoded_string, unsigned char** text){
+    /* Allocates *text, returns a 'new reference'. Caller is required to free */
+
     PyObject* bytes = NULL;
+    Py_ssize_t len = 0;
+    char *buffer;
 
     *text = NULL;
 
     if (PyUnicode_CheckExact(encoded_string)){
         bytes = PyUnicode_AsLatin1String(encoded_string);
+        PyBytes_AsStringAndSize(bytes, &buffer, &len);
     } else if (PyBytes_Check(encoded_string)) {
-        bytes = encoded_string;
+        PyBytes_AsStringAndSize(encoded_string, &buffer, &len);
     }
-    if (bytes) {
-        *text = (unsigned char*)PyBytes_AsString(bytes);
+
+    if (len) {
+        *text = calloc(len,1);
+        if (*text) {
+            memcpy(*text, buffer, len);
+        }
+        if(bytes) {
+            Py_DECREF(bytes);
+        }
         return;
     }
 
+
 #if PY_VERSION_HEX < 0x03000000
     /* likely case here is py2.x with an ordinary string.
        but this isn't defined in Py3.x */
     if (PyString_Check(encoded_string)) {
-        *text = (unsigned char *)PyString_AsString(encoded_string);
+        PyString_AsStringAndSize(encoded_string, &buffer, &len);
+        *text = calloc(len,1);
+        if (*text) {
+            memcpy(*text, buffer, len);
+        }
+        return;
     }
+        
 #endif
 }
 
@@ -2195,6 +2266,7 @@ _font_getmask(ImagingFontObject* self, PyObject* args)
 
     im = ImagingNew(self->bitmap->mode, textwidth(self, text), self->ysize);
     if (!im) {
+        free(text);
         return NULL;
     }
 
@@ -2220,9 +2292,11 @@ _font_getmask(ImagingFontObject* self, PyObject* args)
         x = x + glyph->dx;
         b = b + glyph->dy;
     }
+    free(text);
     return PyImagingNew(im);
 
   failed:
+    free(text);
     ImagingDelete(im);
     return NULL;
 }
@@ -2232,6 +2306,7 @@ _font_getsize(ImagingFontObject* self, PyObject* args)
 {
     unsigned char* text;
     PyObject* encoded_string;
+    PyObject* val;
 
     if (!PyArg_ParseTuple(args, "O:getsize", &encoded_string))
         return NULL;
@@ -2241,7 +2316,9 @@ _font_getsize(ImagingFontObject* self, PyObject* args)
         return NULL;
     }
 
-    return Py_BuildValue("ii", textwidth(self, text), self->ysize);
+    val = Py_BuildValue("ii", textwidth(self, text), self->ysize);
+    free(text);
+    return val;
 }
 
 static struct PyMethodDef _font_methods[] = {
@@ -2940,7 +3017,6 @@ static struct PyMethodDef methods[] = {
     {"convert_matrix", (PyCFunction)_convert_matrix, 1},
     {"convert_transparent", (PyCFunction)_convert_transparent, 1},
     {"copy", (PyCFunction)_copy, 1},
-    {"copy2", (PyCFunction)_copy2, 1},
     {"crop", (PyCFunction)_crop, 1},
     {"expand", (PyCFunction)_expand_image, 1},
     {"filter", (PyCFunction)_filter, 1},
@@ -2972,6 +3048,7 @@ static struct PyMethodDef methods[] = {
 
     {"getband", (PyCFunction)_getband, 1},
     {"putband", (PyCFunction)_putband, 1},
+    {"split", (PyCFunction)_split, 1},
     {"fillband", (PyCFunction)_fillband, 1},
 
     {"setmode", (PyCFunction)im_setmode, 1},
@@ -3056,21 +3133,10 @@ _getattr_ptr(ImagingObject* self, void* closure)
 static PyObject*
 _getattr_unsafe_ptrs(ImagingObject* self, void* closure)
 {
-    return Py_BuildValue("(ss)(si)(si)(si)(si)(si)(sn)(sn)(sn)(sn)(sn)(si)(si)(sn)",
-                         "mode", self->image->mode,
-                         "type", self->image->type,
-                         "depth", self->image->depth,
-                         "bands", self->image->bands,
-                         "xsize", self->image->xsize,
-                         "ysize", self->image->ysize,
-                         "palette", self->image->palette,
+    return Py_BuildValue("(sn)(sn)(sn)",
                          "image8", self->image->image8,
                          "image32", self->image->image32,
-                         "image", self->image->image,
-                         "block", self->image->block,
-                         "pixelsize", self->image->pixelsize,
-                         "linesize", self->image->linesize,
-                         "destroy", self->image->destroy
+                         "image", self->image->image
                         );
 };
 
@@ -3255,6 +3321,164 @@ static PyTypeObject PixelAccess_Type = {
 
 /* -------------------------------------------------------------------- */
 
+static PyObject*
+_get_stats(PyObject* self, PyObject* args)
+{
+    PyObject* d;
+    ImagingMemoryArena arena = &ImagingDefaultArena;
+
+    if (!PyArg_ParseTuple(args, ":get_stats"))
+        return NULL;
+
+    d = PyDict_New();
+    if ( ! d)
+        return NULL;
+    PyDict_SetItemString(d, "new_count",
+                         PyInt_FromLong(arena->stats_new_count));
+    PyDict_SetItemString(d, "allocated_blocks",
+                         PyInt_FromLong(arena->stats_allocated_blocks));
+    PyDict_SetItemString(d, "reused_blocks",
+                         PyInt_FromLong(arena->stats_reused_blocks));
+    PyDict_SetItemString(d, "reallocated_blocks",
+                         PyInt_FromLong(arena->stats_reallocated_blocks));
+    PyDict_SetItemString(d, "freed_blocks",
+                         PyInt_FromLong(arena->stats_freed_blocks));
+    PyDict_SetItemString(d, "blocks_cached",
+                         PyInt_FromLong(arena->blocks_cached));
+    return d;
+}
+
+static PyObject*
+_reset_stats(PyObject* self, PyObject* args)
+{
+    ImagingMemoryArena arena = &ImagingDefaultArena;
+
+    if (!PyArg_ParseTuple(args, ":reset_stats"))
+        return NULL;
+    
+    arena->stats_new_count = 0;
+    arena->stats_allocated_blocks = 0;
+    arena->stats_reused_blocks = 0;
+    arena->stats_reallocated_blocks = 0;
+    arena->stats_freed_blocks = 0;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject*
+_get_alignment(PyObject* self, PyObject* args)
+{
+    if (!PyArg_ParseTuple(args, ":get_alignment"))
+        return NULL;
+    
+    return PyInt_FromLong(ImagingDefaultArena.alignment);
+}
+
+static PyObject*
+_get_block_size(PyObject* self, PyObject* args)
+{
+    if (!PyArg_ParseTuple(args, ":get_block_size"))
+        return NULL;
+    
+    return PyInt_FromLong(ImagingDefaultArena.block_size);
+}
+
+static PyObject*
+_get_blocks_max(PyObject* self, PyObject* args)
+{
+    if (!PyArg_ParseTuple(args, ":get_blocks_max"))
+        return NULL;
+    
+    return PyInt_FromLong(ImagingDefaultArena.blocks_max);
+}
+
+static PyObject*
+_set_alignment(PyObject* self, PyObject* args)
+{
+    int alignment;
+    if (!PyArg_ParseTuple(args, "i:set_alignment", &alignment))
+        return NULL;
+    
+    if (alignment < 1 || alignment > 128) {
+        PyErr_SetString(PyExc_ValueError, "alignment should be from 1 to 128");
+        return NULL;
+    }
+    /* Is power of two */
+    if (alignment & (alignment - 1)) {
+        PyErr_SetString(PyExc_ValueError, "alignment should be power of two");
+        return NULL;
+    }
+
+    ImagingDefaultArena.alignment = alignment;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject*
+_set_block_size(PyObject* self, PyObject* args)
+{
+    int block_size;
+    if (!PyArg_ParseTuple(args, "i:set_block_size", &block_size))
+        return NULL;
+    
+    if (block_size <= 0) {
+        PyErr_SetString(PyExc_ValueError,
+            "block_size should be greater than 0");
+        return NULL;
+    }
+
+    if (block_size & 0xfff) {
+        PyErr_SetString(PyExc_ValueError,
+            "block_size should be multiple of 4096");
+        return NULL;
+    }
+
+    ImagingDefaultArena.block_size = block_size;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject*
+_set_blocks_max(PyObject* self, PyObject* args)
+{
+    int blocks_max;
+    if (!PyArg_ParseTuple(args, "i:set_blocks_max", &blocks_max))
+        return NULL;
+    
+    if (blocks_max < 0) {
+        PyErr_SetString(PyExc_ValueError,
+            "blocks_max should be greater than 0");
+        return NULL;
+    }
+
+    if ( ! ImagingMemorySetBlocksMax(&ImagingDefaultArena, blocks_max)) {
+        ImagingError_MemoryError();
+        return NULL;
+    }
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject*
+_clear_cache(PyObject* self, PyObject* args)
+{
+    int i = 0;
+
+    if (!PyArg_ParseTuple(args, "|i:clear_cache", &i))
+        return NULL;
+    
+    ImagingMemoryClearCache(&ImagingDefaultArena, i);
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+/* -------------------------------------------------------------------- */
+
 /* FIXME: this is something of a mess.  Should replace this with
    pluggable codecs, but not before PIL 1.2 */
 
@@ -3272,6 +3496,7 @@ extern PyObject* PyImaging_PackbitsDecoderNew(PyObject* self, PyObject* args);
 extern PyObject* PyImaging_PcdDecoderNew(PyObject* self, PyObject* args);
 extern PyObject* PyImaging_PcxDecoderNew(PyObject* self, PyObject* args);
 extern PyObject* PyImaging_RawDecoderNew(PyObject* self, PyObject* args);
+extern PyObject* PyImaging_SgiRleDecoderNew(PyObject* self, PyObject* args);
 extern PyObject* PyImaging_SunRleDecoderNew(PyObject* self, PyObject* args);
 extern PyObject* PyImaging_TgaRleDecoderNew(PyObject* self, PyObject* args);
 extern PyObject* PyImaging_XbmDecoderNew(PyObject* self, PyObject* args);
@@ -3316,12 +3541,10 @@ static PyMethodDef functions[] = {
     {"blend", (PyCFunction)_blend, 1},
     {"fill", (PyCFunction)_fill, 1},
     {"new", (PyCFunction)_new, 1},
-
-    {"getcount", (PyCFunction)_getcount, 1},
+    {"merge", (PyCFunction)_merge, 1},
 
     /* Functions */
     {"convert", (PyCFunction)_convert2, 1},
-    {"copy", (PyCFunction)_copy2, 1},
 
     /* Codecs */
     {"bcn_decoder", (PyCFunction)PyImaging_BcnDecoderNew, 1},
@@ -3351,6 +3574,7 @@ static PyMethodDef functions[] = {
     {"pcx_encoder", (PyCFunction)PyImaging_PcxEncoderNew, 1},
     {"raw_decoder", (PyCFunction)PyImaging_RawDecoderNew, 1},
     {"raw_encoder", (PyCFunction)PyImaging_RawEncoderNew, 1},
+    {"sgi_rle_decoder", (PyCFunction)PyImaging_SgiRleDecoderNew, 1},
     {"sun_rle_decoder", (PyCFunction)PyImaging_SunRleDecoderNew, 1},
     {"tga_rle_decoder", (PyCFunction)PyImaging_TgaRleDecoderNew, 1},
     {"xbm_decoder", (PyCFunction)PyImaging_XbmDecoderNew, 1},
@@ -3409,6 +3633,17 @@ static PyMethodDef functions[] = {
     {"outline", (PyCFunction)PyOutline_Create, 1},
 #endif
 
+    /* Resource management */
+    {"get_stats", (PyCFunction)_get_stats, 1},
+    {"reset_stats", (PyCFunction)_reset_stats, 1},
+    {"get_alignment", (PyCFunction)_get_alignment, 1},
+    {"get_block_size", (PyCFunction)_get_block_size, 1},
+    {"get_blocks_max", (PyCFunction)_get_blocks_max, 1},
+    {"set_alignment", (PyCFunction)_set_alignment, 1},
+    {"set_block_size", (PyCFunction)_set_block_size, 1},
+    {"set_blocks_max", (PyCFunction)_set_blocks_max, 1},
+    {"clear_cache", (PyCFunction)_clear_cache, 1},
+
     {NULL, NULL} /* sentinel */
 };
 
