@@ -109,7 +109,7 @@ getfont(PyObject* self_, PyObject* args, PyObject* kw)
     unsigned char* font_bytes;
     int font_bytes_size = 0;
     static char* kwlist[] = {
-        "filename", "size", "index", "encoding", "font_bytes", 
+        "filename", "size", "index", "encoding", "font_bytes",
         "layout_engine", NULL
     };
 
@@ -121,7 +121,8 @@ getfont(PyObject* self_, PyObject* args, PyObject* kw)
         return NULL;
     }
 
-    if (!PyArg_ParseTupleAndKeywords(args, kw, "eti|iss#i", kwlist,
+    if (!PyArg_ParseTupleAndKeywords(args, kw, "eti|is"PY_ARG_BYTES_LENGTH"i", 
+                                     kwlist,
                                      Py_FileSystemDefaultEncoding, &filename,
                                      &size, &index, &encoding, &font_bytes,
                                      &font_bytes_size, &layout_engine)) {
@@ -224,6 +225,11 @@ text_layout_raqm(PyObject* string, FontObject* self, const char* dir,
     if (PyUnicode_Check(string)) {
         Py_UNICODE *text = PyUnicode_AS_UNICODE(string);
         Py_ssize_t size = PyUnicode_GET_SIZE(string);
+	if (! size) {
+	    /* return 0 and clean up, no glyphs==no size, 
+	       and raqm fails with empty strings */
+	    goto failed;
+	}
         if (!raqm_set_text(rq, (const uint32_t *)(text), size)) {
             PyErr_SetString(PyExc_ValueError, "raqm_set_text() failed");
             goto failed;
@@ -233,6 +239,9 @@ text_layout_raqm(PyObject* string, FontObject* self, const char* dir,
     else if (PyString_Check(string)) {
         char *text = PyString_AS_STRING(string);
         int size = PyString_GET_SIZE(string);
+	if (! size) {
+	    goto failed;
+	}
         if (!raqm_set_text_utf8(rq, text, size)) {
             PyErr_SetString(PyExc_ValueError, "raqm_set_text_utf8() failed");
             goto failed;
@@ -449,8 +458,10 @@ font_getsize(FontObject* self, PyObject* args)
     y_max = y_min = 0;
 
     count = text_layout(string, self, dir, features, &glyph_info, 0);
-    if (count == 0)
+    if (PyErr_Occurred()) {
         return NULL;
+    }
+
 
     for (x = i = 0; i < count; i++) {
         int index, error;
@@ -498,6 +509,11 @@ font_getsize(FontObject* self, PyObject* args)
         FT_Done_Glyph(glyph);
     }
 
+    if (glyph_info) {
+        PyMem_Free(glyph_info);
+        glyph_info = NULL;
+    }
+            
     if (face) {
 
         /* left bearing */
@@ -585,9 +601,12 @@ font_render(FontObject* self, PyObject* args)
 
     glyph_info = NULL;
     count = text_layout(string, self, dir, features, &glyph_info, mask);
-    if (count == 0) {
+    if (PyErr_Occurred()) {
         return NULL;
     }
+    if (count == 0) {
+        Py_RETURN_NONE;
+    }
 
     im = (Imaging) id;
     /* Note: bitmap fonts within ttf fonts do not work, see #891/pr#960 */
