@@ -38,6 +38,7 @@
 #include "Lzw.h"
 #include "Raw.h"
 #include "Bit.h"
+#include "Sgi.h"
 
 
 /* -------------------------------------------------------------------- */
@@ -671,6 +672,39 @@ PyImaging_RawDecoderNew(PyObject* self, PyObject* args)
 }
 
 
+/* -------------------------------------------------------------------- */
+/* SGI RLE                                                              */
+/* -------------------------------------------------------------------- */
+
+PyObject*
+PyImaging_SgiRleDecoderNew(PyObject* self, PyObject* args)
+{
+    ImagingDecoderObject* decoder;
+
+    char* mode;
+    char* rawmode;
+    int ystep = 1;
+    int bpc = 1;
+    if (!PyArg_ParseTuple(args, "ss|ii", &mode, &rawmode, &ystep, &bpc))
+        return NULL;
+
+    decoder = PyImaging_DecoderNew(sizeof(SGISTATE));
+    if (decoder == NULL)
+        return NULL;
+
+    if (get_unpacker(decoder, mode, rawmode) < 0)
+        return NULL;
+
+    decoder->pulls_fd = 1;
+    decoder->decode = ImagingSgiRleDecode;
+    decoder->state.ystep = ystep;
+
+    ((SGISTATE*)decoder->state.context)->bpc = bpc;
+
+    return (PyObject*) decoder;
+}
+
+
 /* -------------------------------------------------------------------- */
 /* SUN RLE                                                              */
 /* -------------------------------------------------------------------- */
@@ -819,6 +853,7 @@ PyImaging_JpegDecoderNew(PyObject* self, PyObject* args)
     char* jpegmode; /* what's in the file */
     int scale = 1;
     int draft = 0;
+
     if (!PyArg_ParseTuple(args, "ssz|ii", &mode, &rawmode, &jpegmode,
                           &scale, &draft))
         return NULL;
@@ -830,6 +865,13 @@ PyImaging_JpegDecoderNew(PyObject* self, PyObject* args)
     if (decoder == NULL)
         return NULL;
 
+    // libjpeg-turbo supports different output formats.
+    // We are choosing Pillow's native format (3 color bytes + 1 padding)
+    // to avoid extra conversion in Unpack.c.
+    if (ImagingJpegUseJCSExtensions() && strcmp(rawmode, "RGB") == 0) {
+        rawmode = "RGBX";
+    }
+
     if (get_unpacker(decoder, mode, rawmode) < 0)
         return NULL;
 
